// Questo codice usa le seguienti librerie
// MQTT.h e PubSubClient.h> - versione 1.99.0 - https://github.com/Imroy/pubsubclient

#include <WiFiClient.h>
#include <ESP8266WiFi.h>
#include <MQTT.h>
#include <PubSubClient.h>
#include "DHT.h"


#define DHTPIN 2
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE);

#define SERIAL_SPEED        9600                                        //Definisco la velocità della seriale

//----WIFI CONFIG ----
#define WIFI_SSID           "miao"                                      //Definisco l'SSID della Wifi
#define WIFI_PASSWD         "12345678"                                  //Definisco la password della Wifi
#define MAX_WIFI_INIT_RETRY 10
#define WIFI_RETRY_DELAY    500

//----MQTT CONFIG ----
#define MQTT_CLIENT_ID      "ARDUINO"                                    //Definisco il nome del sub
#define MQTT_SERVER         "192.168.43.77"                              //Definisco l'indirizzo del broker server MQTT.
//#define MQTT_UNAME          "UNAME"                                    //Esiste la possibilità di definire delle credenziali. In questo esempio saranno omesse.
//#define MQTT_PASSW          "PASSWD"                                   //MQTT broker passowrd
#define MQTT_BROKER_PORT    1883                                         //Porta del broker MQTT in ascolto (il valore 1883 è uno standard)
#define MQTT_TOPIC          "temp"                                       //Definisco il topic, che dovrà essere uguale tra pub e sub per comunicare

#define ESP_NAME            "ESP"                                        //Definisco il nome del modulino wifi
#define ESP_PUB_ROLE

WiFiClient wifi_client;                                                  //Crea un client in grado di connettersi ad un specifico indirizzo Internet IP e porta, come definito nella client.connect ().
PubSubClient mqtt_client(wifi_client, MQTT_SERVER, MQTT_BROKER_PORT);
bool mqtt_status;                                                        //Variabile booleana rappresentativa dello stato della connessione



  /* 
    Inizializzo la connessione Wi-Fi.

    La funzione ritorna lo stato della connessione Wifi, tramite
    Wifi.status(), che restituisce:

    WL_CONNECTED: assegnato quando è collegato ad una WiFi rete;
    WL_NO_SHIELD: assegnato quando nessun WiFi scudo è presente;
    WL_IDLE_STATUS: è uno stato temporaneo assegnato quando WiFi.begin() viene chiamato e rimane attivo fino al numero di tentativi di scadenza (con conseguente WL_CONNECT_FAILED) o una connessione viene stabilita (con conseguente WL_CONNECTED);
    WL_NO_SSID_AVAIL: assegnato quando nessun SSID sono disponibili;
    WL_SCAN_COMPLETED: assegnato quando la scansione delle reti è completato;
    WL_CONNECT_FAILED: assegnato quando la connessione non riesce dopo tutti i tentativi;
    WL_CONNECTION_LOST: assegnato quando la connessione è persa;
    WL_DISCONNECTED: assegnato quando ci si scollega dalla rete;
  */

int WiFi_init()
{
  const char* wifi_ssid   = WIFI_SSID;
  const char* wifi_passwd = WIFI_PASSWD;

  int retries = 0;                                                 //tentativi
  
  Serial.println("Connessione all'AP WiFi..........");
  
  WiFi.mode(WIFI_STA);                                             //(dalla libreria esp8266wifi) Definisco il modo tra WIFI_AP, WIFI_STA, WIFI_AP_STA or WIFI_OFF.
  WiFi.begin(wifi_ssid, wifi_passwd);                              //Avvio la connessione Wifi verso l'AP
  
  //verificare lo stato della connessione WiFi da WL_CONNECTED
  while ((WiFi.status() != WL_CONNECTED) && (retries < MAX_WIFI_INIT_RETRY)) {
    retries++;
    delay(WIFI_RETRY_DELAY);
    Serial.println("#");
  }
  
  Serial.println(String(WiFi.localIP()[0]) + "." + String(WiFi.localIP()[1]) + "." + String(WiFi.localIP()[2]) + "." + String(WiFi.localIP()[3]));
  return WiFi.status();

}

int MQTT_init(boolean topic_subscribe)
{
  Serial.println("Avvio della comunicazione MQTT.........");
  mqtt_client.set_max_retries(255);

  if (mqtt_client.connect(MQTT::Connect(MQTT_CLIENT_ID).set_keepalive(90))) {     //Ci colleghiamo al broker MQTT e aumentiamo il keepalive per una maggiore affidabilità

    Serial.println("Connessione al broker MQTT:    SUCCESS    ..........");

    if (topic_subscribe) {

      if (mqtt_client.subscribe(MQTT_TOPIC)) {
        Serial.println("Sottoscrizione al topic MQTT [" + String(MQTT_TOPIC) + "] SUCCESS.........");
      } else {
        Serial.println("Impossibile sottoscrivere al topic MQTT [" + String(MQTT_TOPIC) + "] ERROR.........");
        mqtt_client.disconnect();
        return false;
      }

    }

  } else {
    Serial.println("Connessione al broker MQTT ERROR..........");
  }
  return mqtt_client.connected();
}

void setup() {
  Serial.begin(SERIAL_SPEED);
  delay(100);
  dht.begin();
  Serial.println();
  Serial.println("Avvio MQTT....");
  while (true) {
    if (WiFi_init() != WL_CONNECTED) {                           //Verifico la connessione
      Serial.println("Connessione WiFi ERROR....");
    } else {
      break;
    }
  }
  Serial.println("Connessione WiFi OK....");
  while (true) {
    #ifdef ESP_PUB_ROLE
        mqtt_status = MQTT_init(false); //Se l'ESP è definito come Publisher, non sottoscrive al topic
    #endif
w
    if (!mqtt_status) {
      Serial.println("Connessione MQTT ERROR....");
    }
    else {
      break;
    }
  }
}

void loop() {
  if (mqtt_status) {
    #ifdef ESP_PUB_ROLE

        int h = dht.readHumidity();
        int t = dht.readTemperature();

        Serial.println(String(h) + "," + String(t));
        delay(5000); // 5 sec (la scelta è arbitraria in funzione delle necessità)

        mqtt_client.publish(MQTT_TOPIC, stringa3);
        Serial.println("Messaggio inviato....");
        delay(3000);

    #endif

  }
}
